package com.ann.demo.interfaces.impl;

import com.alibaba.fastjson.JSON;
import com.ann.demo.entity.constant.AliasName;
import com.ann.demo.entity.constant.InterfaceName;
import com.ann.demo.entity.constant.WebserviceId;
import com.ann.demo.entity.filing.MessageSubordinate;
import com.ann.demo.entity.filing.dto.MessageDto;
import com.ann.demo.entity.interfaceEntity.*;
import com.ann.demo.interfaces.HomepageDictionary;
import com.ann.demo.service.*;
import com.ann.demo.service.impl.AnalysisService;
import com.ann.demo.utils.XMLUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import java.util.Date;
import java.util.Objects;

/**
 * @Author: LeiJiaXin
 * @Date: 2019/6/11 17:01
 */

@WebService(serviceName = "HomepageDictionary", // 与接口中指定的name一致
        targetNamespace = "http://interfaces.demo.ann.com/", // 与接口中的命名空间一致,一般是接口的包名倒
        endpointInterface = "com.ann.demo.interfaces.HomepageDictionary"// 接口地址
)
public class HomepageDictionaryImpl implements HomepageDictionary {

    private static final Logger logger = LoggerFactory.getLogger(HomepageDictionaryImpl.class);

    @Autowired
    private AnalysisService analysisService;

    @Autowired
    private MessageService messageService;

    @Autowired
    private MessageSubordinateService messageSubordinateService;


    @Autowired
    private  ArchiveMasterService archiveMasterService;

    @Autowired
    private InterfaceHisCacheService interfaceHisCacheService;


    @Override
    public String HIPMessageServer(String action, String message) {
        String result = "";
        try {
            if (action != null && message != null && !(message.equals("") && action.equals(""))) {
                switch (action) {
                    case InterfaceName.PARAMETER_ADMISSION:
                        // 入院
                        result = analysisService.getHIPMessageServerResult(message, ArchiveMaster.class, InterfaceName.PARAMETER_ADMISSION, WebserviceId.PARAMETER_ADMISSION);
                        break;
                    case InterfaceName.PARAMETER_DISCHARGE:
                        // 出院
                        result = analysisService.getHIPMessageServerResult(message, ArchiveMaster.class, InterfaceName.PARAMETER_DISCHARGE, WebserviceId.PARAMETER_DISCHARGE);
                        break;
                    case InterfaceName.PARAMETER_TRANSFER:
                        // 转科
                        result = analysisService.getHIPMessageServerResult(message, ArchiveMaster.class, InterfaceName.PARAMETER_TRANSFER, WebserviceId.PARAMETER_TRANSFER);
                        break;
                    case InterfaceName.PARAMETER_CANCELLATION_DISCHARGE:
                        // 取消入院
                        result = analysisService.getHIPMessageServerResult(message, ArchiveMaster.class, InterfaceName.PARAMETER_CANCELLATION_DISCHARGE, WebserviceId.PARAMETER_CANCELLATION_DISCHARGE);
                        break;
                    case InterfaceName.PARAMETER_PATIENT_UPDATE:
                        // 患者信息更新
                        result = analysisService.getHIPMessageServerResult(message, ArchiveMaster.class, InterfaceName.PARAMETER_PATIENT_UPDATE, WebserviceId.PARAMETER_PATIENT_UPDATE);
                        break;
                    case InterfaceName.BED_MANAGER:
                        // 同步主管医生
                        result = analysisService.getHIPMessageServerResult(message, ArchiveMaster.class, InterfaceName.BED_MANAGER, WebserviceId.BED_MANAGER);
                        break;
                    case InterfaceName.DIE_MEDICAL_ADVICE:
                        // 同步死亡医嘱
                        result = analysisService.getHIPMessageServerResult(message, ArchiveMaster.class, InterfaceName.DIE_MEDICAL_ADVICE, WebserviceId.DIE_MEDICAL_ADVICE);
                        break;
                    case InterfaceName.PARAMETER_DISCHARGE_BACK:
                        // 同步召回信息
                        result = analysisService.getHIPMessageServerResult(message, ArchiveMaster.class, InterfaceName.PARAMETER_DISCHARGE_BACK, WebserviceId.PARAMETER_DISCHARGE_BACK);
                        break;
                }
            }
        }catch (Exception e){
            logger.error("当前参数：{},报错信息：{}",action,e.getMessage());
            return  "";
        }
        return result;
    }

    @Override
    public String HandleMedicalAdvice(String xmlStr) { // 手动处理有问题医嘱
        String outContent = "失败";
        String remark = null;
        try {
            // 1、定义变量
            boolean flag = false;
            Integer result = 0;
            String type = "";
            String patientMainStr = null,archiveMasterId = null;
            long startTime = System.currentTimeMillis();
            Date xmlCreateTime = null;
            MessageLog messageLog = new MessageLog(xmlStr, InterfaceName.HANDLE_MEDICAL_ADVICE, new Date(), WebserviceId.HANDLE_MEDICAL_ADVICE);

            messageService.save(messageLog);
            // 2、  解析
            MessageDto messageDto = XMLUtils.xpathToBean(xmlStr, ArchiveMaster.class, InterfaceName.HANDLE_MEDICAL_ADVICE);
            if (messageDto != null) {
                archiveMasterId = messageService.getArchiveMasterId(messageDto.getInpNo(), messageDto.getVisitId());
                if(archiveMasterId != null){
                    remark = interfaceHisCacheService.handleMedicalAdvice(messageDto.getInpNo(), messageDto.getVisitId(),messageDto.getType(),messageDto.getXmlCreateTime());
                    if (remark == null) {

                        result = 1;
                        flag = true;
                    }
                }else{
                    remark = "没有入院消息";
                }
                xmlCreateTime = messageDto.getXmlCreateTime();
                type = AliasName.HANDLE_MEDICAL_ADVICE;
            } else {
                remark = "传过来的消息无法解析";
            }

            outContent = AnalysisService.getReturnResult(flag,remark);
            //  更新日志表
            long endTime = System.currentTimeMillis();
            messageLog.setValue(archiveMasterId,outContent, result, messageDto == null ? "" : JSON.toJSONString(messageDto), xmlCreateTime, (endTime - startTime), new Date(), type,remark);
            messageService.save(messageLog);
            if (flag) {
                messageSubordinateService.save(new MessageSubordinate(messageLog.getId(), patientMainStr, JSON.toJSONString(messageLog)));
            }
        }catch (Exception e){
            try {
                logger.error("当前参数：{},报错信息：{}","HandleMedicalAdvice",e.getMessage());
                return AnalysisService.getReturnResult(false,remark);
            } catch (Exception ex) {
                logger.error("当前参数：{},报错信息：{}","HandleMedicalAdvice",ex.getMessage());
            }
        }
        return outContent;
    }


}
