package com.ann.demo.repository;

import com.ann.demo.entity.interfaceEntity.ArchiveMaster;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @Author: LeiJiaXin
 * @Date: 2019/7/11 20:55
 */
@Repository
public interface ArchiveMasterRepository extends JpaRepository<ArchiveMaster, String> {

    @Query("select o.id from ArchiveMaster o where  o.visitId = ?1 and o.patientId =?2 and o.inpNo =?3 and isValid = 0 and archivestate <> '作废' ")
    public String findArchiveMasterId( String visitId, String patientId,String inpNo);

    public ArchiveMaster findArchiveMasterById(String id);


    @Transactional
    @Modifying
    @Query("update ArchiveMaster o set o.archiveState = '作废',o.isValid = 1 where o.id = ?1 and o.isValid = 0 ")
    public Integer updateArchiveMasterValid(String id);

    @Transactional
    @Modifying
    @Query("update ArchiveMaster o set o.transferDept = ?2 where o.id = ?1  and  o.isValid = 0 and archivestate <> '作废'  ")
    public Integer transferDepartment(String id, String transferDept);

    public ArchiveMaster findByIdAndIsValid(String id, Integer isValid);

    public ArchiveMaster findByPatientIdAndVisitIdAndIsValid(String patientId, String visitId, Integer isValid);

    public ArchiveMaster findByInpNoAndVisitIdAndIsValid(String inpNo, String visitId, int isValid);

    @Query("select o from ArchiveMaster o where o.checkedDatetime is not null and  o.isValid = 0")
    public List<ArchiveMaster> findAllArchiveMaster();

    @Transactional
    @Modifying
    @Query("update ArchiveMaster o set o.checkedDatetime = null where o.id = ?1 and o.isValid = 0 and archivestate <> '作废' ")
    public Integer updateCheckedDatetime(String id);

    @Query("select o.transferDept from ArchiveMaster o where o.id = ?1 and o.isValid = 0 and archivestate <> '作废'")
    String findTransferDept(String id);

    @Transactional
    @Modifying
    @Query("update ArchiveMaster o set o.name = ?3,o.sex = ?2 where o.id = ?1 and o.isValid = 0 and archivestate <> '作废' ")
    Integer updateArchiveMaster(String id, String sex, String name);

    @Transactional
    @Modifying
    @Query("update ArchiveMaster o set o.deathFlag = ?2 where o.id = ?1 and o.isValid = 0 and archivestate <> '作废' ")
    Integer updateDeathFlag(String id, String deathFlag);

    @Transactional
    @Modifying
    @Query("update ArchiveMaster o set o.doctorInCharge = ?2 where o.id = ?1 and o.isValid = 0 and archivestate <> '作废' ")
    Integer updateDoctorInCharge(String id, String doctorInCharge);

    @Transactional
    @Modifying
    @Query("update ArchiveMaster o set o.name = ?2 , o.sex = ?3 ,o.admissionDateTime = ?4 ,o.deptAdmissionTo = ?5 where o.id = ?1 and o.isValid = 0 and archivestate <> '作废' ")
    void updateArchiveMasterByUpdate(String id,String name, String sex, Date admissionDateTime, String deptAdmissionTo);
    @Transactional
    @Modifying
    @Query("update ArchiveMaster o set o.archiveState = '在院' , o.cmtDoctor = 0 , o.cmtNurse = 0 where o.id = ?1 and o.isValid = 0 and archivestate not in ('作废','初审','已归档','复审退回') ")
    Integer updateArchiveMasterArchiveState(String id);
}
