package com.ann.demo.service.impl;

import com.alibaba.fastjson.JSON;
import com.ann.demo.entity.constant.AliasName;
import com.ann.demo.entity.constant.InterfaceName;
import com.ann.demo.entity.filing.dto.MessageDto;
import com.ann.demo.entity.interfaceEntity.ArchiveMaster;
import com.ann.demo.entity.interfaceEntity.MessageLog;
import com.ann.demo.entity.normalEntity.OutPara;
import com.ann.demo.service.*;
import com.ann.demo.utils.OracleConnect;
import com.ann.demo.utils.XMLUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 解析逻辑
 *
 * @Author: LeiJiaXin
 * @Date: 2019/8/20 16:29
 */
@Component
public class AnalysisService {


    @Autowired
    MessageService messageService;

    @Autowired
    MessageSubordinateService messageSubordinateService;

    @Autowired
    ArchiveMasterService archiveMasterService;

    /**
     * 2019-7-8
     *
     * @Param xmlStr 需要解析的字符串
     * @Param tClass 被解析字符串的实体类
     * @Param parameter 如果是报告 携带参数
     * 返回返回参数xml
     */
    public String getHIPMessageServerResult(String message, Class tClass, String parameter,String webserviceId) throws Exception{
        long startTime = System.currentTimeMillis();
        String textContent = "成功啦~", type = "", typeCodeValue = "AE", outContent = "", patientMainStr = "";
        Date xmlCreateTime = null;
        Integer result = 0;
        long endTime = 0;
        String archiveMasterId = null;

        MessageLog messageLog = new MessageLog(message, InterfaceName.HIP_MESSAGE_SERVER, new Date(),webserviceId);
        // 只要没有传过 才会插入新的
        messageService.save(messageLog);
        MessageDto messageDto = XMLUtils.xpathToBean(message, tClass, parameter);
        if (messageDto != null) {
            type = messageDto.getType();
            xmlCreateTime = messageDto.getXmlCreateTime();

            String textContentTemp = null;
            if (Objects.equals(messageDto.getEntity(), ArchiveMaster.class.getName())) { // 患者信息
                ArchiveMaster archiveMaster = JSON.parseObject(messageDto.getObject().toString(), ArchiveMaster.class);
                //修改主管医生方法
                setBedManager(archiveMaster,parameter);
                //查询患者是否存在
                archiveMasterId = archiveMasterService.findArchiveMasterId( archiveMaster.getVisitId(), archiveMaster.getPatientId(),archiveMaster.getInpNo());
                if (archiveMasterId != null) {
                    // 如果不是新增 都会查询id
                    archiveMaster.setId(archiveMasterId);
                }
                textContentTemp = changeArchiveMaster(archiveMaster, messageDto.getAddFlag(), messageDto.getType());
            }
            // textContentTemp 1.错误信息 2.archiveMasterId 3.null
            if (textContentTemp != null && textContentTemp.indexOf("-") == -1) {
                textContent = textContentTemp;
            } else {
                //textContentTemp 如果是成功
                if (textContentTemp != null && textContentTemp.indexOf("-") != -1) {
                    archiveMasterId = textContentTemp;
                }
                typeCodeValue = "AA";
                result = 1;
            }
        } else {
            textContent = "传过来的消息无法解析。。";
        }
        outContent = XMLUtils.generateV3XmlStr(typeCodeValue, textContent);

        endTime = System.currentTimeMillis();
        messageLog.setValue(archiveMasterId,outContent, result, messageDto == null ? "" : JSON.toJSONString(messageDto), xmlCreateTime, (endTime - startTime), new Date(), type, textContent);
        messageService.save(messageLog);

        return outContent;
    }

    /**
     *
     * @param archiveMaster
     * @param parameter
     */
    private void setBedManager(ArchiveMaster archiveMaster, String parameter) {
        if (parameter.equals(InterfaceName.BED_MANAGER)){
            //查主管医生id
            String inpNo = archiveMaster.getInpNo();
            String visitId = archiveMaster.getVisitId();
            String selectSql = "SELECT ATTENDING_DOCTOR_ID FROM DOCUS.LJJ_V_ATTENDING where  INP_NO= '"+inpNo+"' and VISIT_ID ='"+visitId+"' ";
            List<String> counts = null;
            try {
                counts = OracleConnect.selectList(selectSql);
                String s = counts.get(0);
                archiveMaster.setDoctorInCharge(s);
            } catch (Exception e) {
                e.printStackTrace();
                throw new RuntimeException("审核时查询oracle记录失败");
            }

        }

    }

    private String changeArchiveMaster(ArchiveMaster archiveMaster, String changeStatus, String aliasName) throws Exception{
        String textContent = null;
        try {
            if (changeStatus != null || aliasName != null) {
                String archiveMasterId = archiveMaster.getId();
                if (archiveMasterId != null) {
                    if (aliasName.equals(AliasName.ADMISSION_RECORD) && Objects.equals(changeStatus, "update")) { //入院消息 更新患者住院信息
                        archiveMasterService.save(archiveMaster); //
                    } else if (aliasName.equals(AliasName.CANCELLATION_DISCHARGE) && Objects.equals(changeStatus, "delete")) { // 取消入院
                        archiveMasterService.updateArchiveMasterValid(archiveMasterId);
                    } else if (aliasName.equals(AliasName.TRANSFER_RECORD)) {  // 转科
                        archiveMasterService.transferDepartment(archiveMaster);
                    } else if (aliasName.equals(AliasName.DISCHARGE_RECORD)) {  // 出院
                        archiveMasterService.discharge(archiveMaster);
                    } else if (aliasName.equals(AliasName.PATIENT_UPDATE)) { //患者信息修改
                        if (!archiveMasterService.updateArchiveMaster(archiveMaster)) {
                            textContent = "患者更新失败";
                        }
                    } else if (aliasName.equals(AliasName.DIE_MEDICAL_ADVICE)) { //死亡信息同步
                        String deathFlag = "1";
                        if (Objects.equals(changeStatus, "cancel")) {
                            deathFlag = "0";
                        }
                        archiveMaster.setDeathFlag(deathFlag);
                        if (!archiveMasterService.updateArchiveMasterByDeathFlag(archiveMaster)) {
                            textContent = "死亡信息同步失败";
                        }
                    } else if (aliasName.equals(AliasName.BED_MANAGER)) { //管床医师同步
                        if (!archiveMasterService.updateArchiveMasterBydoctorInCharge(archiveMaster)) {
                            textContent = "管床医师同步失败";
                        }
                    } else if(aliasName.equals(AliasName.PARAMETER_DISCHARGE_BACK)){ //出院召回
                        if (!archiveMasterService.updateArchiveMasterArchiveState(archiveMaster)) {
                            textContent = "出院召回同步失败";
                        }
                    }else {
                        textContent = "信息有错(存在患者入院消息可能已发送)";
                    }
                } else {
                    if (Objects.equals(changeStatus, "new") && aliasName.equals(AliasName.ADMISSION_RECORD)) { //入院消息 新增患者住院信息
                        archiveMasterService.save(archiveMaster);
                        textContent = archiveMaster.getId();
                    } else {
                        textContent = "没有入院信息";
                    }
                }
            } else {
                textContent = "信息解析错误";
            }
        }catch (Exception e){
            return e.getMessage();
        }
        return textContent;
    }

    public static String getReturnResult(boolean resultCode,String remark) throws Exception{
        OutPara outPara = new OutPara();
        if (resultCode) {
            outPara.setRESULT_CODE(true);
            outPara.setRESULT_CONTENT("成功");
        } else {
            outPara.setRESULT_CODE(false);
            outPara.setRESULT_CONTENT(remark);
        }
        return XMLUtils.generateXmlStr(outPara);
    }

}
